@@ -136,7 +136,9 @@ mod test_custom_specs {
         case("custom/bar_sort_x_axis_categorical", 0.001, true),
         case("custom/taxi_dashboard", 0.001, true),
         case("custom/periods_in_formula_output", 0.001, true),
-        case("custom/bin_transform_rounding", 0.001, true)
+        case("custom/bin_transform_rounding", 0.001, true),
+        case("custom/geojson_inline", 0.001, true),
+        case("custom/gh_361", 0.001, true)
     )]
     fn test_image_comparison(spec_name: &str, tolerance: f64, extract_inline_values: bool) {
         println!("spec_name: {spec_name}");
@@ -1188,6 +1190,7 @@ mod test_pre_transform_inline {
             row_limit: None,
             inline_datasets,
             preserve_interactivity: true,
+            ..Default::default()
         };
         let request = PreTransformSpecRequest {
             spec: serde_json::to_string(&inline_spec).unwrap(),
@@ -1341,6 +1344,7 @@ async fn check_pre_transform_spec_from_files(spec_name: &str, tolerance: f64) {
         row_limit: None,
         inline_datasets: vec![],
         preserve_interactivity: true,
+        ..Default::default()
     };
     let request = PreTransformSpecRequest {
         spec: serde_json::to_string(&full_spec).unwrap(),
