@@ -6,10 +6,10 @@ use sqlparser::ast::{
     FunctionArg as SqlFunctionArg, FunctionArg, Ident, ObjectName as SqlObjectName, ObjectName,
     UnaryOperator as SqlUnaryOperator, WindowFrame as SqlWindowFrame,
     WindowFrameBound as SqlWindowBound, WindowFrameUnits as SqlWindowFrameUnits,
-    WindowSpec as SqlWindowSpec,
+    WindowSpec as SqlWindowSpec, WindowType,
 };
 
-use datafusion_expr::expr::{BinaryExpr, Case, Cast};
+use datafusion_expr::expr::{BinaryExpr, Case, Cast, Sort};
 use datafusion_expr::{
     expr, lit, AggregateFunction, Between, BuiltInWindowFunction, BuiltinScalarFunction, Expr,
     ExprSchemable, Operator, WindowFrameBound, WindowFrameUnits, WindowFunction,
@@ -17,7 +17,7 @@ use datafusion_expr::{
 
 use crate::compile::function_arg::ToSqlFunctionArg;
 use crate::compile::order::ToSqlOrderByExpr;
-use crate::dialect::Dialect;
+use crate::dialect::{Dialect, UnorderedRowNumberMode};
 use vegafusion_common::data::scalar::ScalarValueHelpers;
 use vegafusion_common::error::{Result, VegaFusionError};
 
@@ -294,6 +294,20 @@ impl ToSqlExpr for Expr {
                     BuiltinScalarFunction::Factorial => "factorial",
                     BuiltinScalarFunction::Gcd => "gcd",
                     BuiltinScalarFunction::Lcm => "lcm",
+                    BuiltinScalarFunction::ArrayAppend => "array_append",
+                    BuiltinScalarFunction::ArrayConcat => "array_concat",
+                    BuiltinScalarFunction::ArrayDims => "array_dims",
+                    BuiltinScalarFunction::ArrayFill => "array_fill",
+                    BuiltinScalarFunction::ArrayLength => "array_length",
+                    BuiltinScalarFunction::ArrayNdims => "array_ndims",
+                    BuiltinScalarFunction::ArrayPosition => "array_position",
+                    BuiltinScalarFunction::ArrayPositions => "array_positions",
+                    BuiltinScalarFunction::ArrayPrepend => "array_prepend",
+                    BuiltinScalarFunction::ArrayRemove => "array_remove",
+                    BuiltinScalarFunction::ArrayReplace => "array_replace",
+                    BuiltinScalarFunction::ArrayToString => "array_to_string",
+                    BuiltinScalarFunction::Cardinality => "array_cardinality",
+                    BuiltinScalarFunction::TrimArray => "trim_array",
                 };
                 translate_scalar_function(fun_name, args, dialect, schema)
             }
@@ -344,6 +358,33 @@ impl ToSqlExpr for Expr {
                         (win_fn.to_string().to_ascii_lowercase(), supports_frame)
                     }
                     WindowFunction::AggregateUDF(udf) => (udf.name.to_ascii_lowercase(), true),
+                    WindowFunction::WindowUDF(udf) => (udf.name.to_ascii_lowercase(), true),
+                };
+
+                // Handle unordered row_number
+                let order_by = if fun_name == "row_number" && order_by.is_empty() {
+                    match &dialect.unordered_row_number_mode {
+                        UnorderedRowNumberMode::AlternateScalarFunction(alt_fun) => {
+                            return Ok(SqlExpr::Function(SqlFunction {
+                                name: SqlObjectName(vec![Ident::new(alt_fun)]),
+                                args: vec![],
+                                over: None,
+                                distinct: false,
+                                special: false,
+                                order_by: vec![],
+                            }))
+                        }
+                        UnorderedRowNumberMode::OrderByConstant => {
+                            vec![Expr::Sort(Sort {
+                                expr: Box::new(lit(1)),
+                                asc: false,
+                                nulls_first: false,
+                            })]
+                        }
+                        _ => order_by.clone(),
+                    }
+                } else {
+                    order_by.clone()
                 };
 
                 if dialect.aggregate_functions.contains(&fun_name)
@@ -401,11 +442,11 @@ impl ToSqlExpr for Expr {
                     };
 
                     // Process over
-                    let over = SqlWindowSpec {
+                    let over = WindowType::WindowSpec(SqlWindowSpec {
                         partition_by,
                         order_by,
                         window_frame: sql_window_frame,
-                    };
+                    });
 
                     let sql_fun = SqlFunction {
                         name: ObjectName(vec![Ident {
@@ -416,6 +457,7 @@ impl ToSqlExpr for Expr {
                         over: Some(over),
                         distinct: false,
                         special: false,
+                        order_by: Default::default(),
                     };
 
                     Ok(SqlExpr::Function(sql_fun))
@@ -516,6 +558,7 @@ fn translate_scalar_function(
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     } else if let Some(transformer) = dialect.scalar_transformers.get(fun_name) {
         // Supported through AST transformation
@@ -548,6 +591,7 @@ fn translate_aggregate_function(
             over: None,
             distinct,
             special: false,
+            order_by: Default::default(),
         }))
     } else if let Some(transformer) = dialect.aggregate_transformers.get(fun_name) {
         // Supported through AST transformation
@@ -600,6 +644,8 @@ fn aggr_fn_to_name(fun: &AggregateFunction) -> &str {
         AggregateFunction::BitXor => "bit_xor",
         AggregateFunction::BoolAnd => "bool_and",
         AggregateFunction::BoolOr => "bool_or",
+        AggregateFunction::FirstValue => "first_value",
+        AggregateFunction::LastValue => "last_value",
     }
 }
 
