@@ -1,3 +1,4 @@
+use crate::compile::data_type::ToSqlDataType;
 use crate::dialect::Dialect;
 use arrow::datatypes::DataType;
 use datafusion_common::scalar::ScalarValue;
@@ -5,6 +6,7 @@ use sqlparser::ast::{
     Expr as SqlExpr, Function as SqlFunction, FunctionArg as SqlFunctionArg, FunctionArgExpr,
     Ident, ObjectName as SqlObjectName, Value as SqlValue,
 };
+use std::ops::Add;
 use vegafusion_common::error::{Result, VegaFusionError};
 
 pub trait ToSqlScalar {
@@ -156,26 +158,41 @@ impl ToSqlScalar for ScalarValue {
                     over: None,
                     distinct: false,
                     special: false,
+                    order_by: Default::default(),
                 }))
             }
-            ScalarValue::Date32(_) => Err(VegaFusionError::internal(
-                "Date32 cannot be converted to SQL",
-            )),
+            ScalarValue::Date32(v) => date32_to_date(v, dialect),
             ScalarValue::Date64(_) => Err(VegaFusionError::internal(
                 "Date64 cannot be converted to SQL",
             )),
-            ScalarValue::TimestampSecond(_, _) => Err(VegaFusionError::internal(
-                "TimestampSecond cannot be converted to SQL",
-            )),
-            ScalarValue::TimestampMillisecond(_, _) => Err(VegaFusionError::internal(
-                "TimestampMillisecond cannot be converted to SQL",
-            )),
-            ScalarValue::TimestampMicrosecond(_, _) => Err(VegaFusionError::internal(
-                "TimestampMicrosecond cannot be converted to SQL",
-            )),
-            ScalarValue::TimestampNanosecond(_, _) => Err(VegaFusionError::internal(
-                "TimestampNanosecond cannot be converted to SQL",
-            )),
+            ScalarValue::TimestampSecond(v, _) => {
+                if let Some(v) = v {
+                    Ok(ms_to_timestamp(v * 1000))
+                } else {
+                    Ok(SqlExpr::Value(SqlValue::Null))
+                }
+            }
+            ScalarValue::TimestampMillisecond(v, _) => {
+                if let Some(v) = v {
+                    Ok(ms_to_timestamp(*v))
+                } else {
+                    Ok(SqlExpr::Value(SqlValue::Null))
+                }
+            }
+            ScalarValue::TimestampMicrosecond(v, _) => {
+                if let Some(v) = v {
+                    Ok(ms_to_timestamp(v / 1000))
+                } else {
+                    Ok(SqlExpr::Value(SqlValue::Null))
+                }
+            }
+            ScalarValue::TimestampNanosecond(v, _) => {
+                if let Some(v) = v {
+                    Ok(ms_to_timestamp(v / 1000000))
+                } else {
+                    Ok(SqlExpr::Value(SqlValue::Null))
+                }
+            }
             ScalarValue::IntervalYearMonth(_) => Err(VegaFusionError::internal(
                 "IntervalYearMonth cannot be converted to SQL",
             )),
@@ -209,3 +226,41 @@ impl ToSqlScalar for ScalarValue {
         }
     }
 }
+
+fn ms_to_timestamp(v: i64) -> SqlExpr {
+    let function_ident = Ident {
+        value: "epoch_ms_to_utc_timestamp".to_string(),
+        quote_style: None,
+    };
+
+    let v_ms_expr = SqlExpr::Value(SqlValue::Number(v.to_string(), false));
+
+    let args = vec![SqlFunctionArg::Unnamed(FunctionArgExpr::Expr(v_ms_expr))];
+
+    SqlExpr::Function(SqlFunction {
+        name: SqlObjectName(vec![function_ident]),
+        args,
+        over: None,
+        distinct: false,
+        special: false,
+        order_by: Default::default(),
+    })
+}
+
+fn date32_to_date(days: &Option<i32>, dialect: &Dialect) -> Result<SqlExpr> {
+    let epoch = chrono::NaiveDate::from_ymd_opt(1970, 1, 1).unwrap();
+    match days {
+        None => Ok(SqlExpr::Cast {
+            expr: Box::new(ScalarValue::Utf8(None).to_sql(dialect)?),
+            data_type: DataType::Date32.to_sql(dialect)?,
+        }),
+        Some(days) => {
+            let date = epoch.add(chrono::Duration::days(*days as i64));
+            let date_str = date.format("%F").to_string();
+            Ok(SqlExpr::Cast {
+                expr: Box::new(ScalarValue::from(date_str.as_str()).to_sql(dialect)?),
+                data_type: DataType::Date32.to_sql(dialect)?,
+            })
+        }
+    }
+}
