@@ -10,7 +10,11 @@ use crate::dialect::transforms::date_part_tz::{
     DatePartTzWithDatePartAndAtTimezoneTransformer, DatePartTzWithExtractAndAtTimezoneTransformer,
     DatePartTzWithFromUtcAndDatePartTransformer,
 };
-use crate::dialect::transforms::date_to_utc_timestamp::DateToUtcTimestampWithCastAndAtTimeZoneTransformer;
+use crate::dialect::transforms::date_to_utc_timestamp::{
+    DateToUtcTimestampClickhouseTransformer, DateToUtcTimestampMySqlTransformer,
+    DateToUtcTimestampSnowflakeTransform, DateToUtcTimestampWithCastAndAtTimeZoneTransformer,
+    DateToUtcTimestampWithCastFunctionAtTransformer, DateToUtcTimestampWithFunctionTransformer,
+};
 use crate::dialect::transforms::date_trunc_tz::{
     DateTruncTzClickhouseTransformer, DateTruncTzSnowflakeTransformer,
     DateTruncTzWithDateTruncAndAtTimezoneTransformer,
@@ -115,6 +119,18 @@ pub enum ValuesMode {
     SelectUnion,
 }
 
+#[derive(Clone, Debug)]
+pub enum UnorderedRowNumberMode {
+    /// `SELECT ROW_NUMBER() OVER ()` is supported
+    Supported,
+
+    /// `SELECT ${alternate}()` is supported as an alternative
+    AlternateScalarFunction(String),
+
+    /// `SELECT ROW_NUMBER() OVER (ORDER BY 1)` is supported
+    OrderByConstant,
+}
+
 #[derive(Clone, Debug)]
 pub struct Dialect {
     /// sqlparser dialect to use to parse queries
@@ -188,6 +204,9 @@ pub struct Dialect {
 
     /// Whether dialect supports multiple columns in the same table that differ only by case
     pub supports_mixed_case_identical_columns: bool,
+
+    /// How `ROW_NUMBER() OVER ()` should be handled
+    pub unordered_row_number_mode: UnorderedRowNumberMode,
 }
 
 impl Default for Dialect {
@@ -217,6 +236,7 @@ impl Default for Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: false,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 }
@@ -305,6 +325,10 @@ impl Dialect {
                     "date_trunc_tz",
                     DateTruncTzWithDateTruncAndAtTimezoneTransformer::new_dyn(false),
                 ),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampWithCastAndAtTimeZoneTransformer::new_dyn(),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -337,6 +361,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -344,6 +369,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: false,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -433,6 +459,10 @@ impl Dialect {
                     "utc_timestamp_to_str",
                     UtcTimestampToStrBigQueryTransformer::new_dyn(),
                 ),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampWithFunctionTransformer::new_dyn("timestamp"),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -463,6 +493,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -470,6 +501,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -532,6 +564,10 @@ impl Dialect {
                 ),
                 ("date_part_tz", DatePartTzClickhouseTransformer::new_dyn()),
                 ("date_trunc_tz", DateTruncTzClickhouseTransformer::new_dyn()),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampClickhouseTransformer::new_dyn(),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -562,6 +598,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -569,6 +606,7 @@ impl Dialect {
             cast_propagates_null: false,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: true,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -672,6 +710,14 @@ impl Dialect {
                     "utc_timestamp_to_str",
                     UtcTimestampToStrDatabricksTransformer::new_dyn(),
                 ),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampWithCastFunctionAtTransformer::new_dyn(
+                        SqlDataType::Timestamp(None, TimezoneInfo::None),
+                        "to_utc_timestamp",
+                        false,
+                    ),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -704,6 +750,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -711,6 +758,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::OrderByConstant,
         }
     }
 
@@ -721,7 +769,24 @@ impl Dialect {
             parse_dialect: ParseDialect::DataFusion,
             quote_style: '"',
             binary_ops: vec![
-                Eq, NotEq, Lt, LtEq, Gt, GtEq, Plus, Minus, Multiply, Divide, Modulo, And, Or,
+                Eq,
+                NotEq,
+                Lt,
+                LtEq,
+                Gt,
+                GtEq,
+                Plus,
+                Minus,
+                Multiply,
+                Divide,
+                Modulo,
+                And,
+                Or,
+                BitwiseAnd,
+                BitwiseOr,
+                BitwiseXor,
+                BitwiseShiftLeft,
+                BitwiseShiftRight,
             ]
             .into_iter()
             .collect(),
@@ -789,6 +854,8 @@ impl Dialect {
                 "covar",
                 "covar_pop",
                 "corr",
+                "q1",
+                "q3",
             ]
             .iter()
             .map(|s| s.to_string())
@@ -848,6 +915,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -860,6 +928,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: true,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -1002,6 +1071,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -1009,6 +1079,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -1072,6 +1143,10 @@ impl Dialect {
                     StrToUtcTimestampMySqlTransformer::new_dyn(),
                 ),
                 ("date_part_tz", DatePartTzMySqlTransformer::new_dyn()),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampMySqlTransformer::new_dyn(),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -1097,6 +1172,7 @@ impl Dialect {
                 (DataType::Float32, SqlDataType::Float(None)),
                 (DataType::Float64, SqlDataType::Double),
                 (DataType::Utf8, SqlDataType::Char(None)),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -1109,6 +1185,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: false,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -1252,6 +1329,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -1259,6 +1337,7 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: true,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -1353,6 +1432,10 @@ impl Dialect {
                     "date_trunc_tz",
                     DateTruncTzWithDateTruncAndAtTimezoneTransformer::new_dyn(true),
                 ),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampWithCastAndAtTimeZoneTransformer::new_dyn(),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -1383,6 +1466,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -1395,6 +1479,7 @@ impl Dialect {
             cast_propagates_null: false,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: false,
+            unordered_row_number_mode: UnorderedRowNumberMode::Supported,
         }
     }
 
@@ -1413,11 +1498,16 @@ impl Dialect {
             parse_dialect: ParseDialect::Snowflake,
             quote_style: '"',
             binary_ops: vec![
-                Eq, NotEq, Lt, LtEq, Gt, GtEq, Plus, Minus, Multiply, Divide, Modulo, And, Or,
+                Eq, NotEq, Lt, LtEq, Gt, GtEq, Plus, Minus, Multiply, Divide, And, Or,
             ]
             .into_iter()
             .collect(),
-            binary_op_transforms: Default::default(),
+            binary_op_transforms: vec![(
+                Modulo,
+                Arc::new(ModulusOpToFunction) as Arc<dyn BinaryOperatorTransformer>,
+            )]
+            .into_iter()
+            .collect(),
             scalar_functions: vec![
                 "abs", "acos", "asin", "atan", "atan2", "ceil", "coalesce", "cos", "exp", "floor",
                 "ln", "pow", "round", "sin", "sqrt", "tan", "trunc", "random", "substr", "concat",
@@ -1490,6 +1580,10 @@ impl Dialect {
                     "utc_timestamp_to_str",
                     UtcTimestampToStrSnowflakeTransformer::new_dyn(),
                 ),
+                (
+                    "date_to_utc_timestamp",
+                    DateToUtcTimestampSnowflakeTransform::new_dyn(),
+                ),
             ]
             .into_iter()
             .map(|(name, v)| (name.to_string(), v))
@@ -1524,6 +1618,7 @@ impl Dialect {
                     DataType::Timestamp(TimeUnit::Millisecond, None),
                     SqlDataType::Timestamp(None, TimezoneInfo::None),
                 ),
+                (DataType::Date32, SqlDataType::Date),
             ]
             .into_iter()
             .collect(),
@@ -1531,6 +1626,9 @@ impl Dialect {
             cast_propagates_null: true,
             supports_non_finite_floats: true,
             supports_mixed_case_identical_columns: true,
+            unordered_row_number_mode: UnorderedRowNumberMode::AlternateScalarFunction(
+                "seq8".to_string(),
+            ),
         }
     }
 }
@@ -1593,6 +1691,7 @@ impl BinaryOperatorTransformer for ModulusOpToFunction {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     }
 }
@@ -1635,6 +1734,7 @@ impl FunctionTransformer for RenameFunctionTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     }
 }
@@ -1666,6 +1766,7 @@ impl FunctionTransformer for ExpWithPowFunctionTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     }
 }
@@ -1704,6 +1805,7 @@ impl FunctionTransformer for CastArgsFunctionTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     }
 }
@@ -1741,6 +1843,7 @@ impl FunctionTransformer for LogBaseTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         }))
     }
 }
@@ -1780,6 +1883,7 @@ impl FunctionTransformer for LogBaseWithLnTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         });
 
         let base_arg = SqlFunctionArg::Unnamed(SqlFunctionArgExpr::Expr(SqlExpr::Value(
@@ -1794,6 +1898,7 @@ impl FunctionTransformer for LogBaseWithLnTransformer {
             over: None,
             distinct: false,
             special: false,
+            order_by: Default::default(),
         });
 
         Ok(SqlExpr::BinaryOp {
@@ -1813,12 +1918,11 @@ impl IsFiniteWithNotInTransformer {
 }
 impl FunctionTransformer for IsFiniteWithNotInTransformer {
     fn transform(&self, args: &[Expr], dialect: &Dialect, schema: &DFSchema) -> Result<SqlExpr> {
-        let isfinite_expr = Expr::InList(expr::InList {
+        let isfinite_expr = !Expr::InList(expr::InList {
             expr: Box::new(args[0].clone()),
             list: vec![lit(f64::NEG_INFINITY), lit(f64::INFINITY), lit(f64::NAN)],
             negated: false,
-        })
-        .not();
+        });
         isfinite_expr.to_sql(dialect, schema)
     }
 }
