@@ -1,4 +1,5 @@
 pub mod connection;
+pub mod dataframe;
 
 use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
@@ -11,7 +12,8 @@ use vegafusion_core::proto::gen::pretransform::pre_transform_extract_warning::Wa
 use vegafusion_core::proto::gen::pretransform::pre_transform_values_warning::WarningType as ValueWarningType;
 use vegafusion_runtime::task_graph::runtime::VegaFusionRuntime;
 
-use crate::connection::PySqlConnection;
+use crate::connection::{PySqlConnection, PySqlDataset};
+use crate::dataframe::PyDataFrame;
 use env_logger::{Builder, Target};
 use pythonize::depythonize;
 use vegafusion_common::data::table::VegaFusionTable;
@@ -40,28 +42,54 @@ pub fn initialize_logging() {
 #[pyclass]
 struct PyVegaFusionRuntime {
     runtime: VegaFusionRuntime,
-    tokio_runtime: Runtime,
+    tokio_runtime_connection: Arc<Runtime>,
+    tokio_runtime_current_thread: Arc<Runtime>,
 }
 
-fn process_inline_datasets(
-    inline_datasets: Option<&PyDict>,
-) -> PyResult<HashMap<String, VegaFusionDataset>> {
-    if let Some(inline_datasets) = inline_datasets {
-        Python::with_gil(|py| -> PyResult<_> {
-            inline_datasets
-                .iter()
-                .map(|(name, pyarrow_table)| {
-                    // We convert to ipc bytes for two reasons:
-                    // - It allows VegaFusionDataset to compute an accurate hash of the table
-                    // - It works around https://github.com/hex-inc/vegafusion/issues/268
-                    let table = VegaFusionTable::from_pyarrow(py, pyarrow_table)?;
-                    let dataset = VegaFusionDataset::from_table_ipc_bytes(&table.to_ipc_bytes()?)?;
-                    Ok((name.to_string(), dataset))
-                })
-                .collect::<PyResult<HashMap<_, _>>>()
-        })
-    } else {
-        Ok(Default::default())
+impl PyVegaFusionRuntime {
+    fn process_inline_datasets(
+        &self,
+        inline_datasets: Option<&PyDict>,
+    ) -> PyResult<(HashMap<String, VegaFusionDataset>, bool)> {
+        let mut any_python_sources = false;
+        if let Some(inline_datasets) = inline_datasets {
+            Python::with_gil(|py| -> PyResult<_> {
+                let vegafusion_dataset_module = PyModule::import(py, "vegafusion.dataset")?;
+                let sql_dataset_type = vegafusion_dataset_module.getattr("SqlDataset")?;
+                let df_dataset_type = vegafusion_dataset_module.getattr("DataFrameDataset")?;
+
+                let imported_datasets = inline_datasets
+                    .iter()
+                    .map(|(name, inline_dataset)| {
+                        let dataset = if inline_dataset.is_instance(sql_dataset_type)? {
+                            any_python_sources = true;
+                            let sql_dataset = PySqlDataset::new(inline_dataset.into_py(py))?;
+                            let df = self
+                                .tokio_runtime_current_thread
+                                .block_on(sql_dataset.scan_table(&sql_dataset.table_name))?;
+                            VegaFusionDataset::DataFrame(df)
+                        } else if inline_dataset.is_instance(df_dataset_type)? {
+                            any_python_sources = true;
+
+                            let df = Arc::new(PyDataFrame::new(inline_dataset.into_py(py))?);
+                            VegaFusionDataset::DataFrame(df)
+                        } else {
+                            // Assume PyArrow Table
+                            // We convert to ipc bytes for two reasons:
+                            // - It allows VegaFusionDataset to compute an accurate hash of the table
+                            // - It works around https://github.com/hex-inc/vegafusion/issues/268
+                            let table = VegaFusionTable::from_pyarrow(py, inline_dataset)?;
+                            VegaFusionDataset::from_table_ipc_bytes(&table.to_ipc_bytes()?)?
+                        };
+
+                        Ok((name.to_string(), dataset))
+                    })
+                    .collect::<PyResult<HashMap<_, _>>>()?;
+                Ok((imported_datasets, any_python_sources))
+            })
+        } else {
+            Ok((Default::default(), false))
+        }
     }
 }
 
@@ -91,7 +119,14 @@ impl PyVegaFusionRuntime {
         };
 
         // Build the tokio runtime
-        let tokio_runtime = tokio_runtime_builder
+        let tokio_runtime_connection = tokio_runtime_builder
+            .enable_all()
+            .thread_stack_size(TOKIO_THREAD_STACK_SIZE)
+            .build()
+            .external("Failed to create Tokio thread pool")?;
+
+        // Create current thread runtime
+        let tokio_runtime_current_thread = tokio::runtime::Builder::new_current_thread()
             .enable_all()
             .thread_stack_size(TOKIO_THREAD_STACK_SIZE)
             .build()
@@ -99,17 +134,19 @@ impl PyVegaFusionRuntime {
 
         Ok(Self {
             runtime: VegaFusionRuntime::new(conn, max_capacity, memory_limit),
-            tokio_runtime,
+            tokio_runtime_connection: Arc::new(tokio_runtime_connection),
+            tokio_runtime_current_thread: Arc::new(tokio_runtime_current_thread),
         })
     }
 
     pub fn process_request_bytes(&self, request_bytes: &PyBytes) -> PyResult<PyObject> {
         let response_bytes = self
-            .tokio_runtime
+            .tokio_runtime_connection
             .block_on(self.runtime.query_request_bytes(request_bytes.as_bytes()))?;
         Python::with_gil(|py| Ok(PyBytes::new(py, &response_bytes).into()))
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn pre_transform_spec(
         &self,
         spec: PyObject,
@@ -118,21 +155,40 @@ impl PyVegaFusionRuntime {
         row_limit: Option<u32>,
         preserve_interactivity: Option<bool>,
         inline_datasets: Option<&PyDict>,
+        keep_signals: Option<Vec<(String, Vec<u32>)>>,
+        keep_datasets: Option<Vec<(String, Vec<u32>)>>,
     ) -> PyResult<(PyObject, PyObject)> {
-        let inline_datasets = process_inline_datasets(inline_datasets)?;
+        let (inline_datasets, any_py_sources) = self.process_inline_datasets(inline_datasets)?;
+
         let spec = parse_json_spec(spec)?;
         let preserve_interactivity = preserve_interactivity.unwrap_or(false);
 
-        let (spec, warnings) = self
-            .tokio_runtime
-            .block_on(self.runtime.pre_transform_spec(
-                &spec,
-                &local_tz,
-                &default_input_tz,
-                row_limit,
-                preserve_interactivity,
-                inline_datasets,
-            ))?;
+        // Build keep_variables
+        let mut keep_variables: Vec<ScopedVariable> = Vec::new();
+        for (name, scope) in keep_signals.unwrap_or_default() {
+            keep_variables.push((Variable::new_signal(&name), scope))
+        }
+        for (name, scope) in keep_datasets.unwrap_or_default() {
+            keep_variables.push((Variable::new_data(&name), scope))
+        }
+
+        // Get runtime based on whether there were any Python data sources
+        // (in which case we need to use the current thread tokio runtime to avoid deadlocking)
+        let rt = if any_py_sources {
+            &self.tokio_runtime_current_thread
+        } else {
+            &self.tokio_runtime_connection
+        };
+
+        let (spec, warnings) = rt.block_on(self.runtime.pre_transform_spec(
+            &spec,
+            &local_tz,
+            &default_input_tz,
+            row_limit,
+            preserve_interactivity,
+            inline_datasets,
+            keep_variables,
+        ))?;
 
         let warnings: Vec<_> = warnings
             .iter()
@@ -155,7 +211,7 @@ impl PyVegaFusionRuntime {
         row_limit: Option<u32>,
         inline_datasets: Option<&PyDict>,
     ) -> PyResult<(PyObject, PyObject)> {
-        let inline_datasets = process_inline_datasets(inline_datasets)?;
+        let (inline_datasets, any_py_sources) = self.process_inline_datasets(inline_datasets)?;
         let spec = parse_json_spec(spec)?;
 
         // Build variables
@@ -168,16 +224,22 @@ impl PyVegaFusionRuntime {
             })
             .collect();
 
-        let (values, warnings) = self
-            .tokio_runtime
-            .block_on(self.runtime.pre_transform_values(
-                &spec,
-                &variables,
-                &local_tz,
-                &default_input_tz,
-                row_limit,
-                inline_datasets,
-            ))?;
+        // Get runtime based on whether there were any Python data sources
+        // (in which case we need to use the current thread tokio runtime to avoid deadlocking)
+        let rt = if any_py_sources {
+            &self.tokio_runtime_current_thread
+        } else {
+            &self.tokio_runtime_connection
+        };
+
+        let (values, warnings) = rt.block_on(self.runtime.pre_transform_values(
+            &spec,
+            &variables,
+            &local_tz,
+            &default_input_tz,
+            row_limit,
+            inline_datasets,
+        ))?;
 
         let warnings: Vec<_> = warnings
             .iter()
@@ -211,27 +273,51 @@ impl PyVegaFusionRuntime {
         })
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn pre_transform_extract(
         &self,
         spec: PyObject,
         local_tz: String,
         default_input_tz: Option<String>,
         preserve_interactivity: Option<bool>,
+        extract_threshold: Option<usize>,
+        extracted_format: Option<String>,
         inline_datasets: Option<&PyDict>,
+        keep_signals: Option<Vec<(String, Vec<u32>)>>,
+        keep_datasets: Option<Vec<(String, Vec<u32>)>>,
     ) -> PyResult<(PyObject, Vec<PyObject>, PyObject)> {
-        let inline_datasets = process_inline_datasets(inline_datasets)?;
+        let (inline_datasets, any_py_sources) = self.process_inline_datasets(inline_datasets)?;
         let spec = parse_json_spec(spec)?;
         let preserve_interactivity = preserve_interactivity.unwrap_or(true);
+        let extract_threshold = extract_threshold.unwrap_or(20);
+        let extracted_format = extracted_format.unwrap_or_else(|| "pyarrow".to_string());
 
-        let (tx_spec, datasets, warnings) =
-            self.tokio_runtime
-                .block_on(self.runtime.pre_transform_extract(
-                    &spec,
-                    &local_tz,
-                    &default_input_tz,
-                    preserve_interactivity,
-                    inline_datasets,
-                ))?;
+        // Get runtime based on whether there were any Python data sources
+        // (in which case we need to use the current thread tokio runtime to avoid deadlocking)
+        let rt = if any_py_sources {
+            &self.tokio_runtime_current_thread
+        } else {
+            &self.tokio_runtime_connection
+        };
+
+        // Build keep_variables
+        let mut keep_variables: Vec<ScopedVariable> = Vec::new();
+        for (name, scope) in keep_signals.unwrap_or_default() {
+            keep_variables.push((Variable::new_signal(&name), scope))
+        }
+        for (name, scope) in keep_datasets.unwrap_or_default() {
+            keep_variables.push((Variable::new_data(&name), scope))
+        }
+
+        let (tx_spec, datasets, warnings) = rt.block_on(self.runtime.pre_transform_extract(
+            &spec,
+            &local_tz,
+            &default_input_tz,
+            preserve_interactivity,
+            extract_threshold,
+            inline_datasets,
+            keep_variables,
+        ))?;
 
         let warnings: Vec<_> = warnings
             .iter()
@@ -251,7 +337,20 @@ impl PyVegaFusionRuntime {
                 .map(|(name, scope, table)| {
                     let name = name.into_py(py);
                     let scope = scope.into_py(py);
-                    let table = table.to_pyarrow(py)?;
+                    let table = match extracted_format.as_str() {
+                        "pyarrow" => table.to_pyarrow(py)?,
+                        "arrow-ipc" => {
+                            PyBytes::new(py, table.to_ipc_bytes()?.as_slice()).to_object(py)
+                        }
+                        "arrow-ipc-base64" => table.to_ipc_base64()?.into_py(py),
+                        _ => {
+                            return Err(PyValueError::new_err(format!(
+                                "Invalid extracted_format: {}",
+                                extracted_format
+                            )))
+                        }
+                    };
+
                     let dataset: PyObject = PyTuple::new(py, &[name, scope, table]).into_py(py);
                     Ok(dataset)
                 })
@@ -283,7 +382,8 @@ impl PyVegaFusionRuntime {
     }
 
     pub fn clear_cache(&self) {
-        self.tokio_runtime.block_on(self.runtime.clear_cache());
+        self.tokio_runtime_connection
+            .block_on(self.runtime.clear_cache());
     }
 
     pub fn size(&self) -> usize {
